# Choose whether to build PPU or SPU package
%{!?_with_ppu:%{!?_with_spu:%{error:You must specify one of --with ppu or --with spu.}}}
%{?_with_ppu:%{?_with_spu:%{error:You cannot specify both --with ppu and --with spu.}}}
%define with_ppu 0%{?_with_ppu:%{!?_with_spu:1}}
%define with_spu 0%{?_with_spu:%{!?_with_ppu:1}}
%define package_prefix %{?_with_ppu:ppu}%{?_with_spu:spu}

%if %{with_ppu}
Summary: GNU binary utilities for the Cell/B.E. PPU
%endif
%if %{with_spu}
Summary: GNU binary utilities for the Cell/B.E. SPU
%endif
Name: %{package_prefix}-binutils
Version: 2.18.50
Release: 21%{dist}
License: GPL
Group: Cell Development Tools
Source0: binutils-%{version}-20080408.tar.bz2
Patch1: tcn4013.diff
Patch2: auto_ovl_fix.diff
Patch3: archive_path.diff
Patch4: stack_reserve.diff
Patch5: overlay_manager.diff
Patch6: spu_gprof.diff
Patch7: stub_branch_over.diff
Patch8: nonovl_stub.diff
Patch9: spugas.diff
Patch10: objdump.diff
Patch11: extern_ea.diff
Patch12: max_ovl_size.diff
Patch13: ovl_fixed.diff
Patch14: toc_call.diff
Patch15: ppc_dynrel.diff

BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
BuildRequires: dejagnu, bison, flex, texinfo
%define _unpackaged_files_terminate_build 1

%description
The GNU binutils version %{version}.

# Don't strip as the host binutils may be unable to handle SPU objects.
%define __os_install_post /usr/lib/rpm/brp-compress

# When building the cross-toolchain, reset prefix directory and use sysroot
%ifnarch ppc ppc64
%define _prefix /opt/cell/toolchain
%define _mandir %{_prefix}/man
%define _infodir %{_prefix}/info
%define with_sysroot --with-sysroot=/opt/cell/sysroot
%endif

%prep
%setup -c -q
test -e src && test ! -e binutils-%{version} && ln -s src binutils-%{version}
test ! -e src && test -e binutils-%{version} && ln -s binutils-%{version} src

%patch1 -p0
%patch2 -p0
%patch3 -p0
%patch4 -p0
%patch5 -p0
%patch6 -p0
%patch7 -p0
%patch8 -p0
%patch9 -p0
%patch10 -p0
%patch11 -p0
%patch12 -p0
%patch13 -p0
%patch14 -p0
%patch15 -p0

# Delete out of date file.
rm -f binutils-%{version}/ld/emultempl/spu_ovl.o

%build

%if %{with_ppu}
%ifarch ppc ppc64
%define configuration --enable-install-libbfd --target=powerpc64-linux --enable-targets=spu
%define target_platform powerpc64-linux
%else
%define configuration --disable-install-libbfd --target=powerpc64-linux --enable-targets=spu
%define target_platform powerpc64-linux
%endif
%endif
%if %{with_spu}
%define configuration --target=spu
%define target_platform spu
%endif

OPT_FLAGS=`echo $RPM_OPT_FLAGS|sed -e 's/-fno-rtti//g' -e 's/-fno-exceptions//g'`
TARGET_OPT_FLAGS="-O2 -g -pipe -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64"
CC="gcc $OPT_FLAGS"

rm -fr obj-%{target_platform}
mkdir obj-%{target_platform}

pushd obj-%{target_platform}
CC="$CC" CFLAGS="$TARGET_OPT_FLAGS" CXXFLAGS="$TARGET_OPT_FLAGS" XCFLAGS="$OPT_FLAGS" TCFLAGS="$TARGET_OPT_FLAGS" \
	../binutils-%{version}/configure --prefix=%{_prefix} --mandir=%{_mandir} --infodir=%{_infodir} \
	%{?with_sysroot} %{configuration} --program-prefix=%{package_prefix}- \
	--disable-shared --disable-nls --disable-werror --enable-64-bit-bfd

make %{?_smp_mflags}
popd


%install
rm -fr $RPM_BUILD_ROOT

pushd obj-%{target_platform}
TARGET_PLATFORM=%{target_platform}
make DESTDIR=$RPM_BUILD_ROOT install
popd

# Fix ups.
rm -f $RPM_BUILD_ROOT%{_prefix}/lib{,64}/libiberty.a

%if %{with_ppu}
# Create ppu32- wrapper scripts
cat > $RPM_BUILD_ROOT%{_prefix}/bin/ppu32-as <<"EOF"
#!/bin/sh
exec $(dirname $0)/%{package_prefix}-as -a32 ${1+"$@"}
EOF
cat > $RPM_BUILD_ROOT%{_prefix}/bin/ppu32-ld <<"EOF"
#!/bin/sh
exec $(dirname $0)/%{package_prefix}-ld -m elf32ppclinux ${1+"$@"}
EOF
cat > $RPM_BUILD_ROOT%{_prefix}/bin/ppu32-embedspu <<"EOF"
#!/bin/sh
exec $(dirname $0)/%{package_prefix}-embedspu -m32 ${1+"$@"}
EOF
chmod 755 $RPM_BUILD_ROOT%{_prefix}/bin/ppu32-*
%endif

# Remove Windows/Novell only man pages
rm -f $RPM_BUILD_ROOT%{_mandir}/man1/%{package_prefix}-{dlltool,nlmconv,windres,windmc}*

# Info and man7 files conflict with host toolchain files, 
# just remove them for now.
rm -rf $RPM_BUILD_ROOT%{_infodir}
rm -rf $RPM_BUILD_ROOT%{_mandir}/man7

# Move the installed libbfd files to avoid conflicts with the system binutils
%if %{with_ppu}
%ifarch ppc ppc64
rm $RPM_BUILD_ROOT%{_prefix}/lib/lib{bfd,opcodes}.la
mv $RPM_BUILD_ROOT%{_prefix}/lib/lib{bfd,opcodes}.a $RPM_BUILD_ROOT%{_prefix}/%{target_platform}/lib
mv $RPM_BUILD_ROOT%{_prefix}/include $RPM_BUILD_ROOT%{_prefix}/%{target_platform}
%endif
%endif

cd ..

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/bin
%dir %{_mandir}
%dir %{_mandir}/man1
%{_prefix}/bin/%{package_prefix}-addr2line
%{_prefix}/bin/%{package_prefix}-ar
%{_prefix}/bin/%{package_prefix}-as
%{_prefix}/bin/%{package_prefix}-c++filt
%{_prefix}/bin/%{package_prefix}-ld
%{_prefix}/bin/%{package_prefix}-nm
%{_prefix}/bin/%{package_prefix}-objcopy
%{_prefix}/bin/%{package_prefix}-objdump
%{_prefix}/bin/%{package_prefix}-ranlib
%{_prefix}/bin/%{package_prefix}-readelf
%{_prefix}/bin/%{package_prefix}-size
%{_prefix}/bin/%{package_prefix}-strings
%{_prefix}/bin/%{package_prefix}-strip
%{_prefix}/bin/%{package_prefix}-gprof
%if %{with_ppu}
%{_prefix}/bin/%{package_prefix}-embedspu
%{_prefix}/bin/ppu32-as
%{_prefix}/bin/ppu32-embedspu
%{_prefix}/bin/ppu32-ld
%endif
%dir %{_prefix}/%{target_platform}
%dir %{_prefix}/%{target_platform}/bin
%dir %{_prefix}/%{target_platform}/lib
%{_prefix}/%{target_platform}/bin/*
%{_prefix}/%{target_platform}/lib/ldscripts
%if %{with_ppu}
%ifarch ppc ppc64
%dir %{_prefix}/%{target_platform}/include
%{_prefix}/%{target_platform}/include/*
%{_prefix}/%{target_platform}/lib/libbfd.a
%{_prefix}/%{target_platform}/lib/libopcodes.a
%endif
%endif
%{_mandir}/man1/%{package_prefix}-addr2line.1*
%{_mandir}/man1/%{package_prefix}-ar.1*
%{_mandir}/man1/%{package_prefix}-as.1*
%{_mandir}/man1/%{package_prefix}-c++filt.1*
%{_mandir}/man1/%{package_prefix}-ld.1*
%{_mandir}/man1/%{package_prefix}-nm.1*
%{_mandir}/man1/%{package_prefix}-objcopy.1*
%{_mandir}/man1/%{package_prefix}-objdump.1*
%{_mandir}/man1/%{package_prefix}-ranlib.1*
%{_mandir}/man1/%{package_prefix}-readelf.1*
%{_mandir}/man1/%{package_prefix}-size.1*
%{_mandir}/man1/%{package_prefix}-strings.1*
%{_mandir}/man1/%{package_prefix}-strip.1*
%{_mandir}/man1/%{package_prefix}-gprof.1*


%changelog
* Wed Aug 13 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-21
- SPU auto-overlay --fixed-size bugfixes and improvements.
- Fix ppc64 multi-toc segfault.  #47023
- Fix ppc dynreloc error.

* Mon Jul 21 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-20
- Kill unwanted undefined symbol error.  #46170

* Mon Jul 21 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-19
- Improve objdump disassembly of overlay sections.

* Fri Jul 4 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-18
- Handle hi16 and lo16 relocs.

* Thu Jun 19 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-17
- Correctly use nonovl_stub when function address escapes.  #45676

* Wed Jun 18 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-16
- Fix overlay stub branch check.  #45677

* Fri Jun 13 2008  Ken Werner <ken.werner@de.ibm.com>  2.18.50-15
- Fix for gprof

* Thu Jun 12 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-14
- Make overlay manager interrupt safe, add ld param --extra-stack-space.

* Wed Jun 11 2008  Ken Werner  <ken.werner@de.ibm.com>  2.18.50-13
- spu-gprof added

* Sat Jun 7 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-12
- Add valid area below sp to stack calculation.

* Fri Jun 6 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-11
- Improved --auto-ovl support.  Match archive:path in linker script.

* Wed Jun 4 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-10
- Fixes for --auto-overlay and --stack-analysis.

* Mon May 19 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-9
- Support for __ea variable definitions.

* Tue Apr 8 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-8
- Rebase on binutils version 20080408.  --auto-overlay support.

* Mon Apr 7 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-7
- Allow non-function _SPUEAR_ symbols.  #43864

* Tue Mar 18 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-6
- Correct last patch.

* Fri Mar 14 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-5
- Fix more stub related potential segfaults.
- Support stubs for jump tables.

* Tue Mar 11 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-4
- Fix PR 42611 ld segfault.

* Tue Mar 4 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-3
- Teach stack analysis to handle multiple sections per function.
- Handle multiple entries into overlay sections using the same section sym.

* Mon Feb 18 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-2
- Generate symlinks so that tar can be either src/ or binutils-%{version}/

* Fri Feb 2 2008  Alan Modra  <amodra@au1.ibm.com>  2.18.50-1
- Rebase on new binutils snapshot.

* Sat Nov 24 2007  Ulrich Weigand  <uweigand@de.ibm.com>  2.18-7
- Split back into ppu-binutils and spu-binutils packages.

* Wed Nov 11 2007  Ulrich Weigand  <uweigand@de.ibm.com>  2.18-6
- Move BuildRoot out of %description so it takes effect.

* Wed Oct 10 2007  Ben Elliston  <bje@au.ibm.com>  2.18-5
- Symlink binutils-2.18 to src so that patches generated from the FSF
  binutils tree will apply easily.

* Mon Sep 24 2007  Alan Modra  <amodra@au1.ibm.com>  2.18-4
- Stack analysis fix.

* Fri Sep 7 2007  Ben Elliston  <bje@au.ibm.com>  2.18-3
- Build with --enable-64-bit-bfd.

* Fri Aug 31 2007  Ben Elliston  <bje@au.ibm.com>  2.18-2
- Rename source package to cell-binutils.

* Thu Aug 30 2007  Ben Elliston  <bje@au.ibm.com>  2.18-1
- Progress to GNU binutils 2.18.
